package com.smppw.analysis.domain.service;

import com.smppw.analysis.domain.dataobject.*;
import com.smppw.analysis.domain.dto.info.FundSimilarParams;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.Indicator;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/8/3 15:42
 * @description 基础信息服务
 */
public interface BaseInfoService {
    /**
     * 常用指数
     *
     * @return /
     */
    List<ValueLabelVO> getCommonIndexList();

    /**
     * 获取排名期
     *
     * @return /
     */
    String getLatestRankRat();

    /**
     * 获取标的排名期，与指标排名比较，取最小的排名期
     *
     * @param refId /
     * @return /
     */
    String getLatestRankRat(String refId);

    /**
     * 获取标的类型
     *
     * @param secId /
     * @return /
     */
    String getSecType(String secId);

    /**
     * 获取类型与标的的对应关系
     *
     * @param secIdList 标的集合
     * @return /
     */
    Map<String, List<String>> getTypeSecMap(List<String> secIdList);

    /**
     * 获取标的id与类型对应关系
     *
     * @param secIdList 标的集合
     * @return /
     */
    Map<String, String> querySecsType(List<String> secIdList);

    /**
     * 获取标的id和名称映射关系
     *
     * @param allSecIdList 标的id集合
     * @return /
     */
    Map<String, String> querySecName(List<String> allSecIdList);

    /**
     * 湖区标的净值频率
     *
     * @param secId 标的id
     * @return /
     */
    Frequency getNavFrequency(String secId);

    /**
     * 获取货币基金业绩列表
     *
     * @param fundId 基金id
     * @return /
     */
    List<MonetaryFundProfitDO> queryMonetaryFund(String fundId);

    /**
     * 获取标的业绩排名
     *
     * @param rankDate  排名期
     * @param fundId    基金id
     * @param indexIds  指数
     * @param indicator 指标
     * @return /
     */
    List<Map<String, Object>> getFundRank(String rankDate, String fundId, List<String> indexIds, Indicator indicator);

    /**
     * 获取相似产品，区分管理人同策略和市场同策略
     *
     * @param params /
     * @return /
     */
    List<FundSimilarDo> getFundSimilarList(FundSimilarParams params);

    /**
     * 获取公募基金的公告信息
     *
     * @param fundId   公募基金id
     * @param typeList 信息类别list
     * @param title    信息标题
     * @return 公告信息
     */
    List<ManualFundNoticeInfoDO> queryFundAnnounce(String fundId, List<String> typeList, String title);

    /**
     * 获取公募基金投资类信息说明
     *
     * @param fundId 基金id
     * @return 公募基金投资类信息说明
     */
    FundArchivesInfoDO queryFundArchives(String fundId);

    /**
     * 获取基金的申赎费率信息
     *
     * @param fundId 基金id
     * @return 基金的申赎费率信息
     */
    FundFeeDo getFundFee(String fundId);


    /**
     * 获取公募基金费率信息
     *
     * @param fundId 基金id
     * @return 公募基金费率信息
     */
    List<ManualFundFeeDo> getManualFundFee(String fundId);

    /**
     * 获取人员基本信息
     *
     * @param refId 基金id
     * @return 人员基本信息
     */
    List<PersonnelInformationDo> getFundManagerInfo(String refId);

    /**
     * 获取人员任职经历
     *
     * @param personnelIdList 人员id礼列表
     * @return 人员任职经历
     */
    List<PersonnelWorkExperienceDo> listPersonnelWorkExperience(List<String> personnelIdList);

    /**
     * 获取公募基金的基金经理变更历史
     *
     * @param fundId 基金id
     * @return 公募基金的基金经理变更历史
     */
    List<ManualFundManagerChangeDo> listFundManagerChangeHistory(String fundId);

    /**
     * 获取公募基金的基金经理管理的基金信息
     *
     * @param managerIdList 基金经理id列表
     * @return 公募基金的基金经理管理的基金信息
     */
    List<MfManagerFundNumDo> listMfManagerFundNumByManagerId(List<String> managerIdList);

    /**
     * 获取公募基金正在任职的人员基本信息
     *
     * @param refId 基金id
     * @return 公募基金正在任职的人员基本信息
     */
    List<PersonnelInformationDo> getFundPresentManagerInfo(String refId);


    /**
     * 获取基金资产规模
     * @param managerId 基金经理id
     * @return 基金资产规模
     */
    List<ManualFundAssetSizeDo> getAssetSizeList(String managerId);
}
